﻿using Kivii.DataAnnotations;
using Kivii.Lims;
using Kivii.Lims.Entities;
using Kivii.Linq;
using Kivii.OnlineCustomization.Extensions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.OnlineCustomization.Entities
{
    public class OnlineOrder : EntityWithMetadata, IEntityIsThirdReportOrder
    {
        #region field
        [ApiMember(Description = "订单号")]
        public string OrderId { get; set; }

        /// <summary>
        /// 购物车内不同商品根据分类进行分组
        /// </summary>
        [ApiMember(Description = "分类")]
        public string Category { get; set; }

        [ApiMember(Description = "订单总金额")]
        public decimal TotalAmount { get; set; }

        [ApiMember(Description = "所属Kvid")]
        public Guid OwnerKvid { get; set; }

        [ApiMember(Description = "买家单位")]
        public string BuyerCompony { get; set; }

        [ApiMember(Description = "买家名称")]
        public string BuyerName { get; set; }

        [ApiMember(Description = "买家手机号")]
        public string BuyerMobile { get; set; }

        [ApiMember(Description = "省份")]
        public string Province { get; set; }

        [ApiMember(Description = "城市")]
        public string City { get; set; }

        [ApiMember(Description = "区县")]
        public string District { get; set; }

        [ApiMember(Description = "详细地址")]
        public string Address { get; set; }

        [ApiMember(Description = "备注")]
        public string Remark { get; set; }

        #region 创建/更新人
        [ApiMember(Description = "创建人Kvid")]
        public Guid CreatorKvid { get; set; }

        [ApiMember(Description = "创建人")]
        public string CreatorName { get; set; }

        [ApiMember(Description = "更新人Kvid")]
        public Guid UpdaterKvid { get; set; }

        [ApiMember(Description = "更新人")]
        public string UpdaterName { get; set; }
        #endregion

        [Ignore]
        public OrderStatusType StatusType
        {
            get => (OrderStatusType)Status;
            set => Status = (int)value;
        }

        [Ignore]
        public List<OnlineOrderItem> OrderItems { get; set; }
        #endregion

        #region Interface
        public Report OnInstanceThirdReport(Guid ThirdReportOrderKvid)
        {
            return new Report();
        }

        public object OnPullThirdReport()
        {
            var rtns = new RestfulQueryResponse<ThirdReportOrder>();
            var client = new JsonHttpClient(Configs._defaultUrl);
            var conn = KiviiContext.GetOpenedDbConnection<ThirdReportOrder>();
            DateTime maxTranslateTime = conn.Scalar<ThirdReportOrder, DateTime>(o => Sql.Max(o.OperateTime), p => p.ThirdType == typeof(OnlineOrder).FullName);

            try
            {
                var response = client.PullOrders(Configs._token, Configs._secret, maxTranslateTime);

                rtns.Results = new List<ThirdReportOrder>();
                if (response == null) return rtns;
                if (response.Results.IsNullOrEmpty()) return rtns;
                var results = response.Results.AcceptThirdContractOrder(null, null, conn);

                rtns.Results.AddRange(results);
                rtns.Total = rtns.Results.Count;
                var thread = KiviiContext.NewThread(() =>
                {
                    //没有关联报告的 自动执行符合条件的关联
                    OrderExtension.LinkReports();
                });
                thread.Start();
            }
            catch (Exception ex)
            {
                throw ex;
            }

            return rtns;
        }

        public bool OnRejectThirdReport(Guid ThirdReportOrderKvid)
        {
            if (ThirdReportOrderKvid == Guid.Empty) return false;
            var conn = KiviiContext.GetOpenedDbConnection<ThirdReportOrder>();
            var third = conn.SingleById<ThirdReportOrder>(ThirdReportOrderKvid);
            if (third == null) return false;
            if (third.ThirdType != typeof(OnlineOrder).FullName) return false;
            //订单还未受理的才退回 否则不能退回
            if (third.IsAccepted) return false;// third.AutoLinkReports();
            if (third.IsUploaded) return false;
            var client = new JsonHttpClient(Configs._defaultUrl);
            try
            {
                var resp = client.RejectOrder(third.SerialNumber, Configs._token, Configs._secret);
                
                return true;
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        public bool OnUploadThirdReport(Guid ThirdReportOrderKvid)
        {
            if (ThirdReportOrderKvid == Guid.Empty) return false;
            var conn = KiviiContext.GetOpenedDbConnection<ThirdReportOrder>();
            var third = conn.SingleById<ThirdReportOrder>(ThirdReportOrderKvid);
            if (third == null) return false;
            if (third.ThirdType != typeof(OnlineOrder).FullName) return false;
            //订单还未受理的 则自动去找相关报告进行关联
            if (!third.IsAccepted) return false;// third.AutoLinkReports();
            if (third.IsUploaded) return true;
            var thirdDetails = conn.Select<ThirdReportOrderDetail>(o => o.ThirdOrderKvid == third.Kvid);
            if (thirdDetails.IsNullOrEmpty()) return false;
            var thirdDetailDis = new List<ThirdReportOrderDetail>();
            foreach (var item in thirdDetails)
            {
                if (!thirdDetailDis.Exists(o => o.ReportKvid == item.ReportKvid)) thirdDetailDis.Add(item);
                continue;
            }
            if (thirdDetailDis.IsNullOrEmpty()) return false;
            var reports = conn.SelectByIds<Report>(thirdDetailDis.ConvertAll(o => o.ReportKvid));
            if (reports.IsNullOrEmpty()) return false;
            if (reports.Exists(o => o.Status <= (int)ReportStatusType.ReportIssue)) return false;
            //if (reports.Exists(o => o.Status != (int)ReportStatusType.ReportCollected)) return false;

            var client = new JsonHttpClient(Configs._defaultUrl);

            var pdfs = new List<string>();

            foreach (var thirdDetail in thirdDetailDis)
            {
                if (thirdDetail.ReportKvid == Guid.Empty) continue;
                var report = reports.FirstOrDefault(o => o.Kvid == thirdDetail.ReportKvid);
                if (report == null) continue;
                if (report.Status <= (int)ReportStatusType.ReportIssue) continue;
                #region 报告PDF
                var code = report.Kvid.ToString().Substring(0, 4);
                var internalCode = $"{code}/{report.ReportId}";

                pdfs.Add($"{Configs._pdfUrl}/Lims/Report/{internalCode}");
                #endregion

            }

            try
            {
                var respResult = client.UploadOrderPdf(Configs._token, Configs._secret, third.SerialNumber, pdfs);

                third.IsUploaded = true;
                third.AddOnlyProperties(o => o.IsUploaded);
                conn.UpdateOnly(third);
                var log = new EntityLog<ThirdReportOrder>();
                log.OwnerKvid = third.Kvid;
                log.BizId = third.SerialNumber;
                log.BizKvid = third.Kvid;
                log.BizType = third.ThirdType;
                log.Title = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
                log.Type = "报告数据回传";
                log.Summary = $"回传成功";
                log.Remark = $"{respResult.Results.Count}";
                conn.Insert(log);
                return true;
            }
            catch (Exception ex)
            {
                var err = ex.GetResponseStatus();
                var message = (err == null ? ex.Message : err.Message);
                var log = new EntityLog<ThirdReportOrder>();
                log.OwnerKvid = third.Kvid;
                log.BizId = third.SerialNumber;
                log.BizKvid = third.Kvid;
                log.BizType = third.ThirdType;
                log.Title = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
                log.Type = "报告数据回传";
                log.Summary = $"回传失败";
                log.Remark = $"{message}";
                conn.Insert(log);
                return false;
            }
        }
        #endregion
    }

    public class OnlineOrderItem : EntityWithMetadata
    {
        [ApiMember(Description = "订单Kvid")]
        public Guid OrderKvid { get; set; }

        [ApiMember(Description = "商品Kvid")]
        public Guid GoodsKvid { get; set; }

        [ApiMember(Description = "商品分类")]
        public string Category { get; set; }

        [ApiMember(Description = "商品名称")]
        public string GoodsName { get; set; }

        [ApiMember(Description = "SKU编码")]
        public string SkuCode { get; set; }

        [ApiMember(Description = "商品单价")]
        public decimal UnitPrice { get; set; }

        [ApiMember(Description = "购买数量")]
        public int Quantity { get; set; }

        [ApiMember(Description = "商品总价")]
        public decimal TotalPrice { get; set; }

        [ApiMember(Description = "商品规格信息")]
        public string Specifications { get; set; }

        [ApiMember(Description = "商品图片URL")]
        public string ImageUrl { get; set; }

        [ApiMember(Description = "备注")]
        public string Remark { get; set; }

        #region 创建/更新人
        [ApiMember(Description = "创建人Kvid")]
        public Guid CreatorKvid { get; set; }

        [ApiMember(Description = "创建人")]
        public string CreatorName { get; set; }

        [ApiMember(Description = "更新人Kvid")]
        public Guid UpdaterKvid { get; set; }

        [ApiMember(Description = "更新人")]
        public string UpdaterName { get; set; }
        #endregion
    }

    public enum OrderStatusType
    {
        /// <summary>
        /// 不支持的状态
        /// </summary>
        Unsupported = 0,

        /// <summary>
        /// 已创建
        /// </summary>
        Created = 100,

        /// <summary>
        /// 已支付
        /// </summary>
        Paid = 200,

        /// <summary>
        /// 已发货
        /// </summary>
        Shipped = 300,

        /// <summary>
        /// 已完成
        /// </summary>
        Completed = 400,

        /// <summary>
        /// 已取消
        /// </summary>
        Cancelled = 500
    }

}
